"""ISO/IEC 18004:2006 tables and functions implementation"""

import os.path

__revision__ = "$Rev: 1927 $"

MAX_DATA_BITS = [
    128, 224, 352, 512, 688, 864, 992, 1232, 1456, 1728,
    2032, 2320, 2672, 2920, 3320, 3624, 4056, 4504, 5016, 5352,
    5712, 6256, 6880, 7312, 8000, 8496, 9024, 9544, 10136, 10984,
    11640, 12328, 13048, 13800, 14496, 15312, 15936, 16816, 17728, 18672,

    152, 272, 440, 640, 864, 1088, 1248, 1552, 1856, 2192,
    2592, 2960, 3424, 3688, 4184, 4712, 5176, 5768, 6360, 6888,
    7456, 8048, 8752, 9392, 10208, 10960, 11744, 12248, 13048, 13880,
    14744, 15640, 16568, 17528, 18448, 19472, 20528, 21616, 22496, 23648,

    72, 128, 208, 288, 368, 480, 528, 688, 800, 976,
    1120, 1264, 1440, 1576, 1784, 2024, 2264, 2504, 2728, 3080,
    3248, 3536, 3712, 4112, 4304, 4768, 5024, 5288, 5608, 5960,
    6344, 6760, 7208, 7688, 7888, 8432, 8768, 9136, 9776, 10208,

    104, 176, 272, 384, 496, 608, 704, 880, 1056, 1232,
    1440, 1648, 1952, 2088, 2360, 2600, 2936, 3176, 3560, 3880,
    4096, 4544, 4912, 5312, 5744, 6032, 6464, 6968, 7288, 7880,
    8264, 8920, 9368, 9848, 10288, 10832, 11408, 12016, 12656, 13328
    ]


MAX_CODEWORDS = [0, 26, 44, 70, 100, 134, 172, 196, 242,
    292, 346, 404, 466, 532, 581, 655, 733, 815, 901, 991, 1085, 1156,
    1258, 1364, 1474, 1588, 1706, 1828, 1921, 2051, 2185, 2323, 2465,
    2611, 2761, 2876, 3034, 3196, 3362, 3532, 3706
    ]


MATRIX_REMAIN_BIT = [0, 0, 7, 7, 7, 7, 7, 0,
                    0, 0, 0, 0, 0, 0, 3, 3,
                    3, 3, 3, 3, 3, 4, 4, 4,
                    4, 4, 4, 4, 3, 3, 3, 3,
                    3, 3, 3, 0, 0, 0, 0, 0, 0]


class MatrixInfo:
    """ Provides QR Code version and Error Correction Level
    dependent information necessary for creating matrix"""


    def __init__(self, version, ecl):
        path = os.path.join(os.path.split(__file__)[0], 'qrcode_data')

        self.byte_num = (MATRIX_REMAIN_BIT[version] + (MAX_CODEWORDS[version] << 3))

        filename = path + "/qrv" + str(version) + "_"
        filename += str(ecl) + ".dat"
        fhndl = open(filename ,"rb")
        unpack = lambda y: [ord(x) for x in y]
        self.matrix_d = []
        self.matrix_d.append(unpack(fhndl.read(self.byte_num)))
        self.matrix_d.append(unpack(fhndl.read(self.byte_num)))
        self.matrix_d.append(unpack(fhndl.read(self.byte_num)))
        self.format_info = []
        self.format_info.append(unpack(fhndl.read(15)))
        self.format_info.append(unpack(fhndl.read(15)))
        self.rs_ecc_codewords = ord(fhndl.read(1))
        self.rs_block_order = unpack(fhndl.read(128))
        fhndl.close()

        filename = path + "/rsc" + str(self.rs_ecc_codewords) + ".dat"

        fhndl = open(filename, "rb")

        self.rs_cal_table = []

        for _ in range(0, 256):
            self.rs_cal_table.append(unpack(fhndl.read(self.rs_ecc_codewords)))
        # end for
        fhndl.close()

        filename = path + "/qrvfr" + str(version) + ".dat"
        fhndl = open(filename, "rb")
        frame_data_str = fhndl.read(65535)
        self.frame_data = []
        for line in frame_data_str.split("\n"):
            frame_line = []
            for char in line:
                if char == '1':
                    frame_line.append(1)
                elif char == '0':
                    frame_line.append(0)
                else:
                    raise ValueError("Corrupted frame data file")
                # end if
            # end for
            self.frame_data.append(frame_line)
        # end for
        fhndl.close()
    # end def __init__

    def create_matrix(self, version, codewords):
        """Create matrix based on version and fills it w/ codewords"""

        mtx_size = 17 + (version << 2)
        matrix = [[0 for i in range(mtx_size)] for j in range(mtx_size)]

        max_codewords = MAX_CODEWORDS[version]
        i = 0
        while i < max_codewords:
            codeword_i = codewords[i]
            j = 7
            while j >= 0:
                codeword_bits_number = (i << 3) +  j
                pos_x = self.matrix_d[0][codeword_bits_number]
                pos_y = self.matrix_d[1][codeword_bits_number]
                mask = self.matrix_d[2][codeword_bits_number]
                matrix[pos_x][pos_y] = ((255*(codeword_i & 1)) ^ mask )
                codeword_i = codeword_i >> 1
                j -= 1
            # end while
            i += 1
        # end while

        for matrix_remain in range(MATRIX_REMAIN_BIT[version], 0, -1):
            remain_bit_temp = matrix_remain + ( max_codewords << 3) - 1
            pos_x = self.matrix_d[0][remain_bit_temp]
            pos_y = self.matrix_d[1][remain_bit_temp]
            mask = self.matrix_d[2][remain_bit_temp]
            matrix[pos_x][pos_y] = ( 255 ^ mask )
        # end for
        return matrix
    # end def create_matrix


    def put_format_info(self, matrix, format_info_value):
        """Put format information into the matrix"""

        format_info = ["101010000010010", "101000100100101",
                        "101111001111100", "101101101001011",
                        "100010111111001", "100000011001110",
                        "100111110010111", "100101010100000",
                        "111011111000100", "111001011110011",
                        "111110110101010", "111100010011101",
                        "110011000101111", "110001100011000",
                        "110110001000001", "110100101110110",
                        "001011010001001", "001001110111110",
                        "001110011100111", "001100111010000",
                        "000011101100010", "000001001010101",
                        "000110100001100", "000100000111011",
                        "011010101011111", "011000001101000",
                        "011111100110001", "011101000000110",
                        "010010010110100", "010000110000011",
                        "010111011011010", "010101111101101"]

        format_info_x1 = [0, 1, 2, 3, 4, 5, 7, 8, 8, 8, 8, 8, 8, 8, 8]
        format_info_y1 = [8, 8, 8, 8, 8, 8, 8, 8, 7, 5, 4, 3, 2, 1, 0]
        for i in range(15):
            content = int(format_info[format_info_value][i]) * 255
            matrix[format_info_x1[i]][format_info_y1[i]] = content
            matrix[self.format_info[0][i]][self.format_info[1][i]] = content
        # end for
    # end def put_format_info


    def finalize(self, matrix_content, mask_content):
        """Create final matrix and put frame data into it"""

        mtx_size = len(matrix_content)
        matrix = [[0 for i in range(mtx_size)] for j in range(mtx_size )]

        for i in range(mtx_size):
            for j in range(mtx_size):
                if (int(matrix_content[j][i]) & mask_content) != 0:
                    matrix[i][j] = 1
                else:
                    matrix[i][j] = self.frame_data[i][j]
                # end if
            # end for
        # end for
        return matrix
    # end def finalize


    def calc_demerit_score(self, bit_r, dem_data):
        """Calculate demerit score"""

        n1_search = (chr(255) * 5) + "+|" + (bit_r * 5) + "+"
        n3_search = bit_r + chr(255) + bit_r * 3 + chr(255) + bit_r

        import re
        demerit = [0, 0, 0, 0]
        demerit[2] = len(re.findall(n3_search, dem_data[0])) * 40
        demerit[3] = dem_data[1].count(bit_r) * len(bit_r) * 100
        demerit[3] /= self.byte_num
        demerit[3] -= 50
        demerit[3] = abs(int(demerit[3] / 5)) * 10

        ptn_temp = re.findall(bit_r + bit_r + "+", dem_data[2])
        demerit[1] += sum([len(x) - 1 for x in ptn_temp])

        ptn_temp = re.findall(chr(255) + chr(255) + "+", dem_data[3])
        demerit[1] += sum([len(x) - 1 for x in ptn_temp])
        demerit[1] *= 3

        ptn_temp = re.findall(n1_search, dem_data[0])
        demerit[0] += sum([len(x) - 2 for x in ptn_temp])
        return sum(demerit)
    # end def calc_demerit_score


    def calc_mask_number(self, matrix_content):
        """Calculate mask number for matrix"""

        mtx_size = len(matrix_content)
        mask_number = 0
        min_demerit_score = 0
        hor_master = ""
        ver_master = ""
        for i in range(0, mtx_size):
            for k in range(0, mtx_size):
                hor_master += chr(matrix_content[k][i])
                ver_master += chr(matrix_content[i][k])
            # end for
        # end for

        for i in range(0, 8):
            bit_r = chr((~(1 << i)) & 255)
            bit_mask = chr(1 << i) * mtx_size * mtx_size
            dem_data = ["", "", "", ""]
            dem_data[0] = strings_and(hor_master, bit_mask)
            dem_data[1] = strings_and(ver_master, bit_mask)
            dem_data[2] = strings_and(((chr(170) * mtx_size) + dem_data[1]),
                                (dem_data[1] + (chr(170) * mtx_size)))
            dem_data[3] = strings_or(((chr(170) * mtx_size) + dem_data[1]),
                                (dem_data[1] + (chr(170) * mtx_size)))
            dem_data = [string_not(x) for x in dem_data]

            str_split = lambda x, a: [x[p:p+a] for p in range(0, len(x), a)]
            dem_data = [chr(170).join(str_split(x, mtx_size)) for x in dem_data]

            dem_data[0] += chr(170) + dem_data[1]
            demerit_score = self.calc_demerit_score(bit_r, dem_data)
            if (demerit_score <= min_demerit_score or i == 0):
                mask_number = i
                min_demerit_score = demerit_score
            # end if
        # end for
        return mask_number
    # end def calc_mask_number
# end class MatrixInfo


def strings_and(str1, str2):
    """Apply logical 'and' to strings"""

    if (len(str1) < len(str2)):
        str1, str2 = str2, str1
    # end if
    str2 += '\0' * (len(str1) - len(str2))
    return "".join([chr(ord(x1) & ord(x2)) for x1, x2 in zip(str1, str2)])
# end def strings_and


def strings_or(str1, str2):
    """Apply logical 'or' to strings"""

    if (len(str1) < len(str2)):
        str1, str2 = str2, str1
    # end if
    str2 += '\0' * (len(str1) - len(str2))
    return "".join([chr(ord(x1) | ord(x2)) for x1, x2 in zip(str1, str2)])
# end def strings_or


def string_not(str1):
    """Apply logical 'not' to every symbol of string"""

    return "".join([chr(256 + ~ord(x)) for x in str1])
# end def string_not




import cPickle

byte_num = 359
matrix_d = cPickle.loads("""(lp1
(lp2
I24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI24
aI23
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI22
aI21
aI20
aI19
aI20
aI19
aI20
aI19
aI20
aI19
aI20
aI19
aI20
aI19
aI20
aI19
aI20
aI19
aI20
aI19
aI20
aI19
aI20
aI19
aI18
aI17
aI18
aI17
aI18
aI17
aI18
aI17
aI18
aI17
aI18
aI17
aI18
aI17
aI18
aI17
aI18
aI17
aI18
aI17
aI18
aI17
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI15
aI15
aI15
aI15
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI16
aI15
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI14
aI13
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI12
aI11
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI10
aI9
aI8
aI7
aI8
aI7
aI8
aI7
aI8
aI7
aI8
aI7
aI8
aI7
aI8
aI7
aI8
aI7
aI5
aI4
aI5
aI4
aI5
aI4
aI5
aI4
aI5
aI4
aI5
aI4
aI5
aI4
aI5
aI4
aI3
aI2
aI3
aI2
aI3
aI2
aI3
aI2
aI3
aI2
aI3
aI2
aI3
aI2
aI3
aI2
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aa(lp3
I24
aI24
aI23
aI23
aI22
aI22
aI21
aI21
aI20
aI20
aI19
aI19
aI18
aI18
aI17
aI17
aI16
aI16
aI15
aI15
aI14
aI14
aI13
aI13
aI12
aI12
aI11
aI11
aI10
aI10
aI9
aI9
aI9
aI9
aI10
aI10
aI11
aI11
aI12
aI12
aI13
aI13
aI14
aI14
aI15
aI15
aI16
aI16
aI17
aI17
aI18
aI18
aI19
aI19
aI20
aI20
aI21
aI21
aI22
aI22
aI23
aI23
aI24
aI24
aI24
aI24
aI23
aI23
aI22
aI22
aI21
aI21
aI15
aI15
aI14
aI14
aI13
aI13
aI12
aI12
aI11
aI11
aI10
aI10
aI9
aI9
aI9
aI9
aI10
aI10
aI11
aI11
aI12
aI12
aI13
aI13
aI14
aI14
aI15
aI15
aI21
aI21
aI22
aI22
aI23
aI23
aI24
aI24
aI24
aI24
aI23
aI23
aI22
aI22
aI21
aI21
aI20
aI19
aI18
aI17
aI16
aI15
aI15
aI14
aI14
aI13
aI13
aI12
aI12
aI11
aI11
aI10
aI10
aI9
aI9
aI8
aI8
aI7
aI7
aI5
aI5
aI4
aI4
aI3
aI3
aI2
aI2
aI1
aI1
aI0
aI0
aI0
aI0
aI1
aI1
aI2
aI2
aI3
aI3
aI4
aI4
aI5
aI5
aI7
aI7
aI8
aI8
aI9
aI9
aI10
aI10
aI11
aI11
aI12
aI12
aI13
aI13
aI14
aI14
aI15
aI15
aI16
aI16
aI17
aI17
aI18
aI18
aI19
aI19
aI20
aI20
aI21
aI21
aI22
aI22
aI23
aI23
aI24
aI24
aI24
aI24
aI23
aI23
aI22
aI22
aI21
aI21
aI20
aI20
aI19
aI19
aI18
aI18
aI17
aI17
aI16
aI16
aI15
aI15
aI14
aI14
aI13
aI13
aI12
aI12
aI11
aI11
aI10
aI10
aI9
aI9
aI8
aI8
aI7
aI7
aI5
aI5
aI4
aI4
aI3
aI3
aI2
aI2
aI1
aI1
aI0
aI0
aI0
aI0
aI1
aI1
aI2
aI2
aI3
aI3
aI4
aI4
aI5
aI5
aI7
aI7
aI8
aI8
aI9
aI9
aI10
aI10
aI11
aI11
aI12
aI12
aI13
aI13
aI14
aI14
aI15
aI15
aI16
aI16
aI17
aI17
aI18
aI18
aI19
aI19
aI20
aI20
aI21
aI21
aI22
aI22
aI23
aI23
aI24
aI24
aI16
aI16
aI15
aI15
aI14
aI14
aI13
aI13
aI12
aI12
aI11
aI11
aI10
aI10
aI9
aI9
aI9
aI9
aI10
aI10
aI11
aI11
aI12
aI12
aI13
aI13
aI14
aI14
aI15
aI15
aI16
aI16
aI16
aI16
aI15
aI15
aI14
aI14
aI13
aI13
aI12
aI12
aI11
aI11
aI10
aI10
aI9
aI9
aI9
aI9
aI10
aI10
aI11
aI11
aI12
aI12
aI13
aI13
aI14
aI14
aI15
aI15
aI16
aI16
aa(lp4
I255
aI98
aI100
aI81
aI231
aI90
aI124
aI129
aI247
aI130
aI100
aI153
aI239
aI114
aI116
aI65
aI247
aI74
aI108
aI145
aI231
aI146
aI116
aI137
aI255
aI98
aI100
aI81
aI231
aI90
aI124
aI129
aI96
aI141
aI19
aI118
aI88
aI149
aI227
aI110
aI128
aI133
aI219
aI118
aI112
aI157
aI3
aI102
aI72
aI133
aI243
aI126
aI144
aI149
aI203
aI102
aI96
aI141
aI19
aI118
aI88
aI149
aI227
aI110
aI243
aI114
aI128
aI137
aI203
aI130
aI112
aI145
aI96
aI129
aI3
aI74
aI88
aI81
aI243
aI114
aI128
aI137
aI203
aI130
aI112
aI145
aI124
aI129
aI231
aI90
aI100
aI81
aI255
aI98
aI116
aI137
aI231
aI146
aI108
aI145
aI124
aI129
aI231
aI90
aI100
aI81
aI255
aI98
aI227
aI110
aI88
aI149
aI19
aI118
aI96
aI141
aI102
aI149
aI126
aI133
aI102
aI112
aI157
aI219
aI118
aI128
aI133
aI227
aI110
aI88
aI149
aI19
aI118
aI96
aI141
aI203
aI102
aI144
aI149
aI72
aI133
aI3
aI102
aI112
aI157
aI219
aI118
aI128
aI133
aI227
aI110
aI243
aI114
aI88
aI81
aI3
aI74
aI96
aI129
aI219
aI146
aI144
aI153
aI72
aI65
aI19
aI90
aI112
aI145
aI203
aI130
aI128
aI137
aI243
aI114
aI88
aI81
aI3
aI74
aI96
aI129
aI219
aI146
aI144
aI153
aI227
aI98
aI72
aI65
aI19
aI90
aI112
aI145
aI203
aI130
aI128
aI137
aI243
aI114
aI255
aI98
aI100
aI81
aI231
aI90
aI124
aI129
aI247
aI130
aI100
aI153
aI239
aI114
aI116
aI65
aI247
aI74
aI108
aI145
aI231
aI146
aI116
aI137
aI255
aI98
aI100
aI81
aI231
aI90
aI124
aI129
aI247
aI130
aI100
aI153
aI116
aI65
aI247
aI74
aI108
aI145
aI231
aI146
aI116
aI137
aI255
aI98
aI227
aI110
aI128
aI133
aI219
aI118
aI112
aI157
aI3
aI102
aI72
aI133
aI144
aI149
aI203
aI102
aI96
aI141
aI19
aI118
aI88
aI149
aI227
aI110
aI128
aI133
aI219
aI118
aI112
aI157
aI3
aI102
aI72
aI133
aI243
aI126
aI144
aI149
aI203
aI102
aI96
aI141
aI19
aI118
aI88
aI149
aI227
aI110
aI219
aI146
aI96
aI129
aI3
aI74
aI88
aI81
aI243
aI114
aI128
aI137
aI203
aI130
aI112
aI145
aI129
aI96
aI90
aI19
aI81
aI88
aI98
aI227
aI137
aI128
aI146
aI219
aI145
aI112
aI74
aI3
aI102
aI219
aI157
aI96
aI118
aI3
aI133
aI88
aI110
aI243
aI149
aI128
aI118
aI203
aI141
aI112
aI145
aI124
aI130
aI231
aI137
aI100
aI114
aI255
aI81
aI116
aI74
aI231
aI129
aI108
aI146
aI247
aa.""")
format_info = cPickle.loads("""(lp1
(lp2
I8
aI8
aI8
aI8
aI8
aI8
aI8
aI17
aI18
aI19
aI20
aI21
aI22
aI23
aI24
aa(lp3
I24
aI23
aI22
aI21
aI20
aI19
aI18
aI8
aI8
aI8
aI8
aI8
aI8
aI8
aI8
aa.""")
rs_ecc_codewords = 16
rs_block_order = cPickle.loads("""(lp1
I44
a.""")
from qrcode.data.rsc16 import rs_cal_table
from qrcode.data.fr2 import frame_data



"""Text encoder for code128 barcode encoder"""

__revision__ = "$Rev: 1$"

import logging
log = logging.getLogger("code128")


import encoding

START_A, START_B, START_C = 103, 104, 105
TO_C, TO_B, TO_A = 99, 100, 101

start_codes = \
{ 
    'A': START_A, 
    'B': START_B, 
    'C': START_C 
}

to_values = \
{
    TO_A: START_A,
    TO_B: START_B,
    TO_C: START_C
}

class TextEncoder:
    """Class which encodes a raw text string into a list of 
    character codes.
    Adds in character set switch codes, and compresses pairs of 
    digits under character set C"""
    
    def __init__( self ):
        self.current_charset = 'B'
        self.digits = ""

            
    def switch_charset( self, new_charset ):
        """Switch to a new character set
        Return a single item list containing the switch code"""
        
        log.debug( "Switching charsets from %c to %c",  
                    self.current_charset, new_charset )

        if new_charset == 'A':
            switch_code = self.convert_char('TO_A')
        elif new_charset == 'B':
            switch_code = self.convert_char('TO_B')
        elif new_charset == 'C':
            switch_code = self.convert_char('TO_C')

        self.current_charset = new_charset

        return [switch_code, ]
            

    def switch_charset_if_necessary( self, char, lookahead ):
        """Decide whether we want to switch charsets for the 
        next character"""

        def upcoming_digits( ):
            """Return true if there are more than three consecutive digits
            coming up"""
            num_digits = 0
            for char in lookahead:
                if char.isdigit():
                    num_digits += 1
                else:
                    break

            return num_digits > 3
                    

        codes = []
        if self.current_charset == 'C' and not char.isdigit():
            # Switch from C - the next char is not a digit

            # by default, switch to B
            if char in encoding.charset_b.keys():
                codes = self.switch_charset( 'B' )

            # but if the character's not in B, switch to A
            elif char in encoding.charset_a.keys():
                codes = self.switch_charset( 'A' )

            else:
                log.error( "No charset found for character %d" % ord(char) )

             # Take care of the odd leftover digit if there is one
            if len(self.digits) == 1:
                codes.append( self.convert_char(self.digits[0]) )


        elif self.current_charset == 'B':
            # Do we want to switch from B?

            # Lookahead - are there lots of digits coming up?
            # If so, switch to C
            if upcoming_digits( ):
                codes = self.switch_charset( 'C' )

            # If B can't handle the next char, switch to A
            elif char not in encoding.charset_b.keys():
                if char in encoding.charset_a.keys():
                    codes = self.switch_charset('A')
                else:
                    log.error( "No charset found for character %d" % ord(char) )

        elif self.current_charset == 'A':
            # Do we want to switch from A?

            # Lookahead - are there lots of digits coming up?
            # If so, switch to C
            if upcoming_digits( ):
                codes = self.switch_charset( 'C' )

            # If A can't handle the next char, switch to B
            elif char not in encoding.charset_a.keys():
                if char in encoding.charset_b.keys():
                    codes = self.switch_charset('B')
                else:
                    log.error( "No charset found for character %d" % ord(char) )

        return codes
                

    def convert_char( self, char ):
        """Convert the given character into the current charset
        For A and B and a few cases in C, this is a simple lookup in 
        the charset table.
        For most cases in C, this involves grouping consecutive digits
        into pairs and adding in each pair as a single character"""

        if self.current_charset == 'A':
            return encoding.charset_a[char]

        elif self.current_charset == 'B':
            return encoding.charset_b[char]

        elif self.current_charset == 'C':
            if char in encoding.charset_c.keys():
                return encoding.charset_c[char]
            elif char.isdigit():
                # store char in the digit buffer
                # and append when there are two digits stored
                self.digits += char
                if len(self.digits) == 2:
                    ret = int(self.digits)
                    self.digits = ""
                    return ret

    def optimize_encoding( self, enc ):
        """Perform various optimizations on the encoded string"""

        # [START_X, TO_Y]  => [START_Y,]
        # (This is only relevant at the start)
        # Saves one character
        if enc[1] in to_values.keys():
            enc[0:2] = [to_values[enc[1]]]


    def encode( self, text ):
        """Encode the given text, optimize it and return a 
        list of character codes"""
        
        encoded_text = []

        encoded_text.append( start_codes[self.current_charset] )

        # Start with charset B
        for i, char in enumerate(text):
            encoded_text += self.switch_charset_if_necessary( \
                                    char, text[i:i+10] )

            converted = self.convert_char(char)
            if converted is not None:
                encoded_text.append( converted )
        
        self.optimize_encoding( encoded_text )
        return encoded_text


    def get_bars( self, encoded_text, checksum ):
        """Return the bar encoding (a string of ones and zeroes)
        representing the given encoded text and checksum digit.
        Stop code and termination bars are added onto the end"""
        
        full_code = encoded_text + [checksum, ]
        bars = ""
        for char in full_code:
            bars += encoding.encodings[char]

        bars += encoding.STOP
        bars += "11"
            
        return bars
        



#!/usr/bin/env python
""" Generate python modules from QR Code data files.
Just run it in directory which contains 'qrcode' (QR Code python library)
and 'qrcode_data' subdirs.
"""
import qrcode.isodata as isodata
import cPickle
import os

if not os.path.exists('qrcode/data'):
    os.mkdir('qrcode/data')

INIT = open('qrcode/data/__init__.py', 'w')
INIT.close()

for version in range(1, 41):
    for ecl in range(0, 4):
        byte_num = (isodata.MATRIX_REMAIN_BIT[version] +
                        (isodata.MAX_CODEWORDS[version] << 3))

        filename = "qrcode_data/qrv" + str(version) + "_"
        filename += str(ecl) + ".dat"
        fhndl = open(filename ,"rb")
        unpack = lambda y: [ord(x) for x in y]
        matrix_d = []
        matrix_d.append(unpack(fhndl.read(byte_num)))
        matrix_d.append(unpack(fhndl.read(byte_num)))
        matrix_d.append(unpack(fhndl.read(byte_num)))

        format_info = []
        format_info.append(unpack(fhndl.read(15)))
        format_info.append(unpack(fhndl.read(15)))

        rs_ecc_codewords = ord(fhndl.read(1))
        rs_block_order = unpack(fhndl.read(128))
        fhndl.close()

        qrv = open('qrcode/data/qrv%d_%d.py' % (version, ecl), 'w')
        qrv.write('import cPickle\n\n')
        qrv.write('byte_num = %d\n' % byte_num)
        qrv.write('matrix_d = cPickle.loads("""%s""")\n' %
                                                     cPickle.dumps(matrix_d))
        qrv.write('format_info = cPickle.loads("""%s""")\n' %
                                                  cPickle.dumps(format_info))
        qrv.write('rs_ecc_codewords = %d\n' % rs_ecc_codewords)
        qrv.write('rs_block_order = cPickle.loads("""%s""")\n' %
                                               cPickle.dumps(rs_block_order))
        qrv.write('from qrcode.data.rsc%d import rs_cal_table\n' %
                                                            rs_ecc_codewords)
        qrv.write('from qrcode.data.fr%d import frame_data\n' % version)
        qrv.close()

        filename = "qrcode_data/rsc" + str(rs_ecc_codewords) + ".dat"
        fhndl = open(filename, "rb")

        rs_cal_table = []

        for _ in range(0, 256):
            rs_cal_table.append(unpack(fhndl.read(rs_ecc_codewords)))
        # end for
        fhndl.close()

        rsc = open('qrcode/data/rsc%d.py' % rs_ecc_codewords, 'w')
        rsc.write('import cPickle\n\n')
        rsc.write('rs_cal_table = cPickle.loads("""%s""")\n' %
                                                 cPickle.dumps(rs_cal_table))
        rsc.close()

    filename = "qrcode_data/qrvfr" + str(version) + ".dat"
    fhndl = open(filename, "rb")
    frame_data_str = fhndl.read(65535)
    frame_data = []
    for line in frame_data_str.split("\n"):
        frame_line = []
        for char in line:
            if char == '1':
                frame_line.append(1)
            elif char == '0':
                frame_line.append(0)
            else:
                raise ValueError("Corrupted frame data file")
            # end if
        # end for
        frame_data.append(frame_line)
    # end for
    fhndl.close()
    fr = open('qrcode/data/fr%d.py' % version, 'w')
    fr.write('import cPickle\n\n')
    fr.write('frame_data = cPickle.loads("""%s""")\n' %
                                                   cPickle.dumps(frame_data))
    fr.close()



"""Rendering code for EAN-13 barcode"""

__revision__ = "$Rev: 2025 $"

import Image, ImageFont, ImageDraw
import os

# maps bar width against font size
font_sizes = \
{
    1: 8,
    2: 14,
    3: 18,
    4: 24
}


class EAN13Renderer:
    """Rendering class - given the code and corresponding 
    bar encodings and guard bars,
    it will add edge zones and render to an image"""
    
    def __init__( self, code, left_bars, right_bars, guards ):
        self.code = code
        self.left_bars = left_bars
        self.right_bars = right_bars
        self.guards = guards

    def write_file( self, filename, bar_width ):
        """Write barcode data out to image file
        filename - the name of the image file
        bar_width - the desired width of each bar"""

        def sum_len( total, item ):
            """add the length of a given item to the total"""
            return total + len(item)

        num_bars = (7 * 12) + reduce( sum_len, self.guards, 0 )

        quiet_width = bar_width * 9
        image_width = (2 * quiet_width) + (num_bars * bar_width) 
        image_height = image_width / 2

        img = Image.new( 'L', (image_width,    image_height), 255 )

        class BarWriter:
            """Class which moves across the image, writing out bars""" 
            def __init__(self, img):
                self.img = img
                self.current_x = quiet_width
                self.symbol_top = quiet_width / 2

            def write_bar( self, value, full=False ):
                """Draw a bar at the current position,
                if the value is 1, otherwise move on silently"""

                # only write anything to the image if bar value is 1
                bar_height = int( image_height * (full and 0.9 or 0.8) )
                if value == 1:
                    for ypos in range(self.symbol_top, bar_height):
                        for xpos in range(self.current_x, \
                                            self.current_x+bar_width):
                            img.putpixel( (xpos, ypos), 0 )
                self.current_x += bar_width

            def write_bars( self, bars, full=False ):
                """write all bars to the image"""
                for bar in bars:
                    self.write_bar( int(bar), full )

        # Draw the bars
        writer = BarWriter( img )
        writer.write_bars( self.guards[0], full=True )
        writer.write_bars( self.left_bars )
        writer.write_bars( self.guards[1], full=True )
        writer.write_bars( self.right_bars )
        writer.write_bars( self.guards[2], full=True )
        
        # Draw the text
        font_size = font_sizes.get(bar_width, 24)
            
		# Locate the font file relative to the module
        eandir, _ = os.path.split( __file__ )
        rootdir, _ = os.path.split( eandir )  
        fontfile = os.path.join( rootdir, "fonts", 
                                "courR%02d.pil" % font_size )
        font = ImageFont.load_path( fontfile )
        draw = ImageDraw.Draw( img )
        draw.text( (1*bar_width, int(image_height*.7)), 
                    self.code[0], font=font )
        draw.text( (16*bar_width, int(image_height*.8)),
                    self.code[1:7], font=font )
        draw.text( (63*bar_width, int(image_height*.8)), self.code[7:], font=font )

        img.save( filename )





"""Example code for datamatrix library"""
__revision__ = "$Rev: 1927 $"

from datamatrix import DataMatrixEncoder
import sys
import logging

logging.getLogger("datamatrix").setLevel(logging.DEBUG)
logging.getLogger("datamatrix").addHandler(logging.StreamHandler(sys.stdout))

if __name__ == "__main__":
    encoder = DataMatrixEncoder( sys.argv[1] )
    encoder.save( "test.png" )
    print encoder.get_ascii( )



import cPickle

frame_data = cPickle.loads("""(lp1
(lp2
I1
aI1
aI1
aI1
aI1
aI1
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI1
aI1
aI1
aI1
aI1
aI1
aa(lp3
I1
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI1
aa(lp4
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI1
aI1
aI1
aI0
aI1
aa(lp5
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI1
aI1
aI1
aI0
aI1
aa(lp6
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI1
aI1
aI1
aI0
aI1
aa(lp7
I1
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI1
aa(lp8
I1
aI1
aI1
aI1
aI1
aI1
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI0
aI1
aI1
aI1
aI1
aI1
aI1
aI1
aa(lp9
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp10
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp11
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp12
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp13
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp14
I0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp15
I0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp16
I1
aI1
aI1
aI1
aI1
aI1
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp17
I1
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp18
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp19
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp20
I1
aI0
aI1
aI1
aI1
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp21
I1
aI0
aI0
aI0
aI0
aI0
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp22
I1
aI1
aI1
aI1
aI1
aI1
aI1
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aI0
aa(lp23
a.""")



from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup

# patch distutils if it can't cope with the "classifiers" or
# "download_url" keywords
from sys import version
if version < '2.2.3':
    from distutils.dist import DistributionMetadata
    DistributionMetadata.classifiers = None
    DistributionMetadata.download_url = None
            
hubarcode = setup(name='huBarcode',
      maintainer='Maximillian Dornseif',
      maintainer_email='md@hudora.de',
      url='https://cybernetics.hudora.biz/projects/wiki/huBarcode',
      version='0.51',
      description='generation of barcodes in Python',
      classifiers=['License :: OSI Approved :: BSD License',
                   'Intended Audience :: Developers',
                   'Programming Language :: Python',
                   'Topic :: Multimedia :: Graphics',
                   'Topic :: Office/Business'],
      # download_url
      package_dir = {'huBarcode': ''},
      packages=['huBarcode', 'huBarcode.qrcode', 'huBarcode.qrcode.qrcode_data', 'huBarcode.datamatrix', 'huBarcode.ean13', 'huBarcode.code128'],
      zip_safe=False,
)

from distutils import dir_util,file_util
import sys, os

# from http://www.redbrick.dcu.ie/~noel/distutils.html
# copy fonts into install dir
if "install" in sys.argv:
    wheretoinstall = hubarcode.command_obj['install'].install_purelib
    dir_util.copy_tree('fonts',os.path.join(wheretoinstall,'huBarcode','fonts'))




"""Example code for ean13 library"""
__revision__ = "$Rev: 2025 $"

from ean13 import EAN13Encoder
import sys
import logging

logging.getLogger("ean13").setLevel(logging.DEBUG)
logging.getLogger("ean13").addHandler(logging.StreamHandler(sys.stdout))

if __name__ == "__main__":
    encoder = EAN13Encoder( sys.argv[1] )
    encoder.save( "test.png" )



"""Encoding tables and functions for EAN-13 barcode"""

__revision__ = "$Rev: 1$"

#
# Map the first number system digit to the parity encodings for the
# following fields, in order:
# second number system digit
# the five manufacturing code characters, in order
# 0 = even
# 1 = odd
#
parity_table = \
{
    0: ( 1, 1, 1, 1, 1, 1 ),
    1: ( 1, 1, 0, 1, 0, 0 ),
    2: ( 1, 1, 0, 0, 1, 0 ),
    3: ( 1, 1, 0, 0, 0, 1 ),
    4: ( 1, 0, 1, 1, 0, 0 ),
    5: ( 1, 0, 0, 1, 1, 0 ),
    6: ( 1, 0, 0, 0, 1, 1 ),
    7: ( 1, 0, 1, 0, 1, 0 ),
    8: ( 1, 0, 1, 0, 0, 1 ),
    9: ( 1, 0, 0, 1, 0, 1 ) 
}

#
# How to encode different digits depending on parity and where
# they are in the code
# Values are for digits placed as follows:
# Left hand side, odd parity
# Left hand side, even parity
# Right hand side (all characters)
#
encoding_table = \
{
    0: ("0001101", "0100111", "1110010"),
    1: ("0011001", "0110011", "1100110"),
    2: ("0010011", "0011011", "1101100"),
    3: ("0111101", "0100001", "1000010"),
    4: ("0100011", "0011101", "1011100"),
    5: ("0110001", "0111001", "1001110"),
    6: ("0101111", "0000101", "1010000"),
    7: ("0111011", "0010001", "1000100"),
    8: ("0110111", "0001001", "1001000"),
    9: ("0001011", "0010111", "1110100"),
}

def get_left_encoded( digit, parity ):
    """Get the left hand encoding of the given digit, under
    the given parity (0=even or 1=odd)"""

    if parity not in (0, 1):
        raise Exception("Invalid parity '%s'" % parity)
    elif digit not in range(0, 10):
        raise Exception("Invalid digit '%s'" % digit)
    else:
        return encoding_table[digit][1-parity]

def get_right_encoded( digit ):    
    """Get the right hand encoding of the given digit"""

    if digit not in range(0, 10):
        raise Exception("Invalid digit '%s'" % digit)
    else:
        return encoding_table[digit][2]



#!/usr/bin/env python
 
"""2D Datamatrix barcode encoder

All needed by the user is done via the DataMatrixEncoder class:

>>> encoder = DataMatrixEncoder("HuDoRa")
>>> # encoder.save( "test.png" )
>>> print encoder.get_ascii()
XX  XX  XX  XX  XX  XX  XX  
XX  XXXX  XXXXXX      XXXXXX
XXXXXX    XX          XX    
XXXXXX    XX        XXXX  XX
XXXX  XX  XXXXXX            
XXXXXX    XXXXXXXX    XXXXXX
XX    XX  XXXXXXXX  XXXX    
XX    XX      XXXX      XXXX
XX  XXXXXXXXXX    XXXX      
XX  XXXX    XX            XX
XX  XXXXXX  XXXXXX      XX  
XXXXXX  XX  XX  XX  XX    XX
XX    XX              XX    
XXXXXXXXXXXXXXXXXXXXXXXXXXXX


Implemented by Helen Taylor for HUDORA GmbH.

Detailed documentation on the format here: 
http://grandzebu.net/informatique/codbar-en/datamatrix.htm
Further resources here: http://www.libdmtx.org/resources.php

You may use this under a BSD License.
"""

__revision__ = "$Rev: 1927 $"

from textencoder import TextEncoder
from placement import DataMatrixPlacer
from renderer import DataMatrixRenderer

class DataMatrixEncoder:
    """Top-level class which handles the overall process of 
    encoding input data, placing it in the matrix and 
    outputting the result"""
    
    def __init__( self, text ):
        """Set up the encoder with the input text.
        This will encode the text, 
        and create a matrix with the resulting codewords"""

        enc = TextEncoder( )
        codewords = enc.encode( text )
        
        matrix_size = enc.mtx_size

        self.matrix = [[None, ]*matrix_size for _ in range(0, matrix_size)]

        placer = DataMatrixPlacer( )
        placer.place( codewords, self.matrix )


    def save( self, filename, cellsize=5 ):
        """Write the matrix out to an image file"""

        dmtx = DataMatrixRenderer( self.matrix )
        dmtx.write_file( cellsize, filename )


    def get_imagedata( self, cellsize=5 ):
        """Write the matrix out to an PNG bytestream"""

        dmtx = DataMatrixRenderer( self.matrix )
        return dmtx.get_imagedata( cellsize )


    def get_ascii( self ):
        """Return an ascii representation of the matrix"""
        dmtx = DataMatrixRenderer( self.matrix )
        return dmtx.get_ascii()




"""Example code for code128 library"""
__revision__ = "$Rev: 1$"

from code128 import Code128Encoder
import logging, sys

logging.getLogger("code128").setLevel(logging.DEBUG)
logging.getLogger("code128").addHandler(logging.StreamHandler(sys.stdout))

if __name__ == "__main__":
    encoder = Code128Encoder( sys.argv[1] )
    encoder.save( "test.png" )



#!/usr/bin/env python
# encoding: utf-8
"""
decode.py

Created by Maximillian Dornseif on 2007-06-28.
Copyright (c) 2007 HUDORA GmbH. All rights reserved.
"""

import sys
import os
import unittest
import Image
from operator import itemgetter
import code128encoding as encoding

def widths2bits(widths):
    """ Converts a list of widths into a list of bits.
    
    >>> widths2bits('21234')
    '110110001111'
    >>> widths2bits('01234')
    '1001110000'
    """
    
    ret = []
    statecount = 0
    for width in widths:
        if int(width): # ignore width == 0
            statecount += 1
        ret.append(str(statecount%2)*int(width))
    return ''.join(ret)

def bits2widths(rowbits):
    """Convert a stream of widths to a list of bits.
    
    >>> bits2widths('110110001111')
    '21234'
    >>> bits2widths('11010010000110001010001')
    '2112142311131'
    """
    ret = []
    state = rowbits[0]
    statelen = 0
    for bit in rowbits[1:]:
        statelen += 1
        if bit != state:
            ret.append(str(statelen))
            state = bit
            statelen = 0
    ret.append(str(statelen+1))
    return ''.join(ret)


def bits_to_values(rowbits):
    rowbits = rowbits.strip('0')
    #if len(rowbits) % 11 != 0:
    #    raise RuntimeError, "invalid bitstring %d %d %r" % (len(rowbits), len(rowbits)%11, rowbits)
    while rowbits:
        charbits = rowbits[:11]
        rowbits = rowbits[11:]
        v = encoding.encodings_rev.get(charbits)
        print repr((charbits, v, encoding.charset_a_rev.get(v), encoding.charset_b_rev.get(v), encoding.charset_c_rev.get(v)))

def rawhistogramm(rowbits):
    statecount = {}
    widths = bits2widths(rowbits)
    for width in widths:
        statecount[width] = statecount.get(width, 0) + 1
    return statecount


def normalizedhistogramm(rowbits):
    statecount = rawhistogramm(rowbits)
    top4widths = sorted(statecount.items(), key=itemgetter(1), reverse=True)[:4]
    minwidth = min([int(itemgetter(0)(x)) for x in top4widths])
    return [int(x)/minwidth for x in bits2widths(rowbits)][1:]


class EncodingTests(unittest.TestCase):
    # "HelloWorld!", encoded using "code 128"
    # The space/bar succession is represented by the following widths (space first):
    #02112142311131122142211142211141341113113211341111212412211141412212221221212232331112
    def test_basic(self):
        print bits_to_values(widths2bits('2112142311131122142211142211141341113113211341111212412211141412212221221212232331112'))

class DecodeHelloWorldTests(unittest.TestCase):
    # "HelloWorld!", encoded using "code 128"
    # The space/bar succession is represented by the following widths (space first):
    #02112142311131122142211142211141341113113211341111212412211141412212221221212232331112
    def setUp(self):
        pass
    
    def _get_middle_rowbits(self, filename):
        im = Image.open(filename).convert("1")
        row = im.size[1]/2
        # AND three lines to get arround dittering issues
        for col in range(im.size[0]):
            im.putpixel((col, row), im.getpixel((col, row)) \
                                     & im.getpixel((col, row + 1)) & im.getpixel((col, row + 2)) \
                                     & im.getpixel((col, row - 1)) & im.getpixel((col, row - 2)))
        im.show()
        return [int(im.getpixel((col, row)) == 0) for col in range(im.size[0])]
    
    def test_basic(self):
        rowbits = self._get_middle_rowbits('testdata/HelloWorld.png')
        hist = normalizedhistogramm(rowbits)
        self.assertEqual(''.join([str(x) for x in hist]),
                         '02112142311131122142211142211141341113113211341111212412211141412212221221212232331112')
    
    def test_basic_large(self):
        rowbits = self._get_middle_rowbits('testdata/HelloWorldLarge.png')
        
        widths = bits2widths(rowbits)
        self.assertEqual(rawhistogramm(rowbits), {5: 22, 6: 19, 11: 19, 12: 7, 16: 3, 17: 6, 22: 9, 56: 1})
        
        hist = normalizedhistogramm(rowbits)
        self.assertEqual(''.join([str(x) for x in hist]),
                         '02112142311131122142211142211141341113113211341111212412211141412212221221212232331112')
    
    
    #def test_basic_small(self):
    #    rowbits = self._get_middle_rowbits('testdata/HelloWorldSmall.png')
    #    hist = normalizedhistogramm(rowbits)
    #    self.assertEqual(''.join([str(x) for x in hist]),
    #                     '2112142311131122142211142211141341113113211341111212412211141412212221221212232331112')

if __name__ == '__main__':
    import doctest
    doctest.testmod()
    unittest.main()